/*	$Id: spaceinvaders.h,v 1.72 2017/06/09 19:43:17 hacki Exp $ */

/*
 * Copyright (c) 2017 Marcus Glocker <marcus@nazgul.ch>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * Defines.
 */
#define VERSION				"1.5"
#define FILENAME_DEBUG			"spaceinvaders.log"
#define BUFFER_SIZE			128
#define WAVES				10
#define BOARD_ROWS			24
#define BOARD_COLS			80
#define BOARD_BORDER_LEFT		9
#define BOARD_BORDER_RIGHT		69
#define SPEED_BOARD_UPDATE		10000			   /* 0.01s */
#define SPEED_ALIENS_MOVE		(SPEED_BOARD_UPDATE * 100) /* 1s    */
#define SPEED_ALIENS_MOVE_DEC		(SPEED_BOARD_UPDATE *   3) /* 0.03s */
#define SPEED_ALIENS_MISSILE_MOVE	(SPEED_BOARD_UPDATE *   5) /* 0.05s */
#define SPEED_ALIENS_MISSILE_TRIGGER	(SPEED_BOARD_UPDATE *  50) /* 0.5s  */
#define SPEED_LASER_SHOT_MOVE		(SPEED_BOARD_UPDATE *   4) /* 0.04s */
#define SPEED_MYSTERY_MOVE		(SPEED_BOARD_UPDATE *  20) /* 0.2s  */
#define SPEED_MYSTERY_TRIGGER		(SPEED_BOARD_UPDATE * 100) /* 1s    */
#define SPEED_TEXT_CLEANUP		(SPEED_BOARD_UPDATE * 400) /* 4s    */
#define MYSTERY_LASER_TOP		23
#define PAUSE_SEC			3
#define PLAYER_LIVES			3
#define ALIENS_SHOT_MAX			3
#define ALIENS_PER_LINE			11
#define ALIENS_NUM			55
#define SCORE_EXTRA_LIVE		1500
#define SPRITES_NUM			15
#define D(l, m...)			do { if ((l) <= debug_level) \
					    debug_write(m); } while (0)

/*
 * Structures.
 */
struct sprite {
	char	graph;
	int	score;
	int	damage;
	int	protection;
};

struct control {
	int	wave;
	char	player_name[USERNAME_MAXLEN+1];
	int	player_time;
	int	player_score;
	int	player_hiscore;
	int	player_hiscore_new;
	int	player_lives;
	int	player_pos_x;
	int	player_shot_inprogress;
	int	player_laser_cycle;
	int	player_laser_counter;
	int	aliens_num;
	int	aliens_shot_inprogress;
	int	aliens_dir_now;
	int	aliens_dir_last;
	int	aliens_move_cycle;
	int	aliens_move_usec;
	int	aliens_missile_trigger_cycle;
	int	aliens_missile_cycle;
	int	aliens_speed;
	int	mystery_trigger_cycle;
	int	mystery_move_cycle;
	int	mystery_inprogress;
	int	mystery_laser_top;
	int	mystery_dir;
	int	msg_cleanup_cycle;
	int	score_extra_live;
	int	game_start;
	int	game_pause;
};

/*
 * Enumerations.
 */
enum {
	MODE_USAGE_OPTIONS,
	MODE_USAGE_HELP,
	MODE_USAGE_VERSION,
	MODE_CTRL_INIT_FULL,
	MODE_CTRL_INIT_PARTIAL,
	MODE_BOARD_INIT_FULL,
	MODE_BOARD_INIT_HEAD
};

enum {
	BBS_BACKUP,
	BBS_RESTORE,
	BBS_REMOVE_ALIENS,
	BBS_REMOVE_ALIENS_MISSILE
};

enum {
	DIRECTION_RIGHT,
	DIRECTION_LEFT,
	DIRECTION_DOWN
};

enum {
	ALIENS_SPEED_NO_CHANGE,
	ALIENS_SPEED_FASTER,
	ALIENS_SPEED_VERY_FAST
};

enum {
	SPACE,
	LASER_BASE,
	LASER_SHOT,
	SHIELD_3,
	SHIELD_2,
	SHIELD_1,
	ALIEN_1,
	ALIEN_2,
	ALIEN_3,
	ALIEN_1_MISSILE,
	ALIEN_2_MISSILE,
	ALIEN_3_MISSILE,
	MYSTERY_SHIP,
	FLOOR,
	EXPLOSION
};

/*
 * Prototypes.
 */
void		usage(int);
void		timer_setup(void);
void		timer_clear(void);
void		keyboard_handler(void);
void		hiscore_load(void);
void		hiscore_save(void);
void		ctrl_init(int);
void		screen_intro(void);
void		screen_game_over(void);
void		screen_board_init(int);
void		board_draw(void);
void		board_write(int, int, const char *, ...);
void		board_write_msg(const char *, ...);
void		board_update(int);
void		board_update_player_move(int);
void		board_update_player_laser_trigger(void);
void		board_update_player_laser(void);
int		board_update_aliens_move(void);
int		board_update_aliens_missile_trigger(void);
int		board_update_aliens_missile(void);
void		board_update_mystery_move(void);
void		board_update_mystery_trigger(void);
void		board_backing_store(int);
int		lookup_sprite(char);
int		count_sprites(int, int, int); /* Not used yet. */
void		player_score(int);
void		player_destroy(void);
void		player_redraw_lives(void);
void		game_reset(int);
void		game_pause(void);
char		getkey(void);
void		flush_stdin(void);
void		debug_setup(char *);
void		debug_write(char *, ...);
int		fortune(int);
long long	ati(char *, int, int);
